@@ -5,6 +5,8 @@ from scaled.cluster.scheduler import SchedulerProcess
 from scaled.io.config import (
     DEFAULT_HEARTBEAT_INTERVAL_SECONDS,
     DEFAULT_FUNCTION_RETENTION_SECONDS,
+    DEFAULT_IO_THREADS,
+    DEFAULT_MAX_NUMBER_OF_TASKS_WAITING,
     DEFAULT_WORKER_TIMEOUT_SECONDS,
     DEFAULT_GARBAGE_COLLECT_INTERVAL_SECONDS,
     DEFAULT_TRIM_MEMORY_THRESHOLD_BYTES,
@@ -21,6 +23,8 @@ class SchedulerClusterCombo:
         self,
         address: str,
         n_workers: int,
+        io_threads: int = DEFAULT_IO_THREADS,
+        max_number_of_tasks_waiting: int = DEFAULT_MAX_NUMBER_OF_TASKS_WAITING,
         heartbeat_interval_seconds: int = DEFAULT_HEARTBEAT_INTERVAL_SECONDS,
         event_loop: str = "builtin",
         worker_timeout_seconds: int = DEFAULT_WORKER_TIMEOUT_SECONDS,
@@ -44,7 +48,8 @@ class SchedulerClusterCombo:
         )
         self._scheduler = SchedulerProcess(
             address=ZMQConfig.from_string(address),
-            stop_event=self._stop_event,
+            io_threads=io_threads,
+            max_number_of_tasks_waiting=max_number_of_tasks_waiting,
             per_worker_queue_size=per_worker_queue_size,
             worker_timeout_seconds=worker_timeout_seconds,
             function_retention_seconds=function_retention_seconds,
@@ -56,12 +61,12 @@ class SchedulerClusterCombo:
 
     def __del__(self):
         self.shutdown()
-        logging.info(f"{self.__get_prefix()} shutdown")
 
     def shutdown(self):
+        logging.info(f"{self.__get_prefix()} shutdown")
         self._stop_event.set()
         self._cluster.join()
-        self._scheduler.join()
+        self._scheduler.terminate()
 
     def __get_prefix(self):
         return f"{self.__class__.__name__}:"
