@@ -1,14 +1,17 @@
 import os
 
-POLLING_TIME_MILLISECONDS = 20
+POLLING_TIME_MILLISECONDS = 1000
+CLEANUP_INTERVAL_SECONDS = 1
 
 # scheduler
-DEFAULT_WORKER_TIMEOUT_SECONDS = 30
+DEFAULT_IO_THREADS = 1
+DEFAULT_MAX_NUMBER_OF_TASKS_WAITING = -1  # -1 means no limits
+DEFAULT_WORKER_TIMEOUT_SECONDS = 60
 
 # worker
 DEFAULT_NUMBER_OF_WORKER = os.cpu_count() - 1
-DEFAULT_HEARTBEAT_INTERVAL_SECONDS = 1
+DEFAULT_HEARTBEAT_INTERVAL_SECONDS = 2
 DEFAULT_FUNCTION_RETENTION_SECONDS = 3600
 DEFAULT_GARBAGE_COLLECT_INTERVAL_SECONDS = 30
 DEFAULT_TRIM_MEMORY_THRESHOLD_BYTES = 1024 * 1024 * 1024
-DEFAULT_PER_WORKER_QUEUE_SIZE = 1000
\ No newline at end of file
+DEFAULT_PER_WORKER_QUEUE_SIZE = 1000
